{
   muu TrueColor library for TMT Pascal - Nova 16bpp converter routines
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

PROCEDURE Nx86_S16_D32(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRRRGGG GGGBBBBB -> ________ RRRRR___ GGGGGG__ BBBBB___

ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width

@@pixel_loop:
    xor  eax,  eax
    
    // ax = RRRRRGGG GGGBBBBB, bl = GGGBBBBB
    mov  ax,   [esi]
    mov  bl,   al

    // bl = BBBBB___
    shl  bl,   3

    // ax = RRRRR___ GGGGGG__
    shr  ax,   3
    shl  ah,   3
    and  ax,   $F8FC

    // eax = ________ RRRRR___ GGGGGG__ BBBBB___
    shl  eax,  8
    mov  al,   bl

    mov  [edi],  eax

    add  esi,  2
    add  edi,  4
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S16_D24(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRRRGGG GGGBBBBB -> RRRRR___ GGGGGG__ BBBBB___

ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width

@@pixel_loop:
    xor  eax,  eax
    
    // ax = RRRRRGGG GGGBBBBB, bl = GGGBBBBB
    mov  ax,   [esi]
    mov  bl,   al

    // bl = BBBBB___
    shl  bl,   3

    // ax = RRRRR___ GGGGGG__
    shr  ax,   3
    shl  ah,   3
    and  ax,   $F8FC

    mov  [edi],  bl
    mov  [edi+1],  ax

    add  esi,  2
    add  edi,  3
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S16_D15(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRRRGGG GGgBBBBB -> _RRRRRGG GGGBBBBB
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width

@@pixel_loop:
    // ax = _RRRRRGG GGGGBBBB, bl = GGGBBBBB
    mov  ax,  [esi]
    mov  bl,  al
    shr  ax,  1

    // bl = ___BBBBB
    and  bl,  $1F

    // ax = _RRRRRGG GGGBBBBB
    and  ax,  $7FE0
    or   al,  bl

    mov  [edi],  ax

    add  esi,  2
    add  edi,  2
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S16_D8(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRrrGGg gggBBBbb -> RRRGGBBB
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    // ax = RRRRRGGG GGGBBBBB, bl = RRRRRGGG
    mov  ax,  [esi]
    mov  bl,  ah

    // bl = RRR_____
    and  bl,  $E0

    // ax = ___GG___ _____BBB
    shl  ah,  2
    shr  al,  2
    and  ax,  $1807

    // bl = RRRGGBBB
    or   bl,  al
    or   bl,  ah

    mov  [edi],  bl

    add  esi,  2
    inc  edi
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S16_D32_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRRRGGG GGGBBBBB -> ________ BBBBB___ GGGGGG__ RRRRR___

ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    xor  eax,  eax
    
    // ax = RRRRRGGG GGGBBBBB, bl = GGGBBBBB
    mov  ax,   [esi]
    mov  bl,   al

    // bl = BBBBB___
    shl  bl,   3

    // ax = RRRRR___ GGGGG___
    shr  ax,   3
    shl  ah,   3
    and  ax,   $F8FC

    // eax = ________ BBBBB___ GGGGG___ RRRRR___
    xchg ah,   bl
    shl  eax,  8
    mov  al,   bl

    mov  [edi],  eax

    add  esi,  2
    add  edi,  4
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S16_D24_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRRRGGG GGGBBBBB -> BBBBB___ GGGGGG__ RRRRR___

ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    xor  eax,  eax
    
    // ax = RRRRRGGG GGGBBBBB, bl = GGGBBBBB
    mov  ax,   [esi]
    mov  bl,   al

    // bl = BBBBB___
    shl  bl,   3

    // ax = RRRRR___ GGGGG___
    shr  ax,   3
    shl  ah,   3
    and  ax,   $F8FC

    xchg ah,   bl
    mov  [edi],  bl
    mov  [edi+2],  ax

    add  esi,  2
    add  edi,  3
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S16_D16_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRRRGGG GGGBBBBB -> BBBBBGGG GGGRRRRR
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  ax,  [esi]
    mov  bx,  ax
    
    // bx = ___RRRRR BBBBB___
    shl  bl,  3
    shr  bh,  3

    // ax = _____GGG GGG_____
    and  ax,  $7E0

    // ax = BBBBBGGG GGGRRRRR
    or   ah,  bl
    or   al,  bh

    mov  [edi],  ax

    add  esi,  2
    add  edi,  2
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S16_D15_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRRRGGG GGGBBBBB -> _BBBBBGG GGGRRRRR
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height
    
@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    mov  ax,  [esi]
    mov  bx,  ax
    
    // bx = ___RRRRR _BBBBB__
    shl  bl,  2
    xor  bl,  $7C
    shr  bh,  3

    // ax = ______GG GGG_____
    shr  ax,  1
    and  ax,  $3E0

    // ax = _BBBBBGG GGGRRRRR
    or   ah,  bl
    or   al,  bh

    mov  [edi],  ax

    add  esi,  2
    add  edi,  2
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

PROCEDURE Nx86_S16_D8_SWAP_RB(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRrrGGg gggBBBbb -> BBBGGRRR
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width

@@pixel_loop:
    mov  ax,  [esi]
    mov  bl,  al
    mov  al,  ah

    // bl = BBB____
    shl  bl,  3
    and  bl,  $E0

    // ax = _____RRR ___GG___
    shl  al,  2
    shr  ah,  5
    and  ax,  $718

    // al = BBBGGRRR
    or   al,  ah
    or   al,  bl

    mov  [edi],  al

    add  esi,  2
    inc  edi
    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;
PROCEDURE Nx86_S16_D8_GRAY(src,tgt:pointer;wdt,hgt,snl,tnl:dword); assembler;
    // RRRRRGGG GGGBBBBB -> II______ (sort of..)
ASM
    mov  esi, src   // source
    mov  edi, tgt   // target
    mov  edx, hgt   // height

@@line_loop:
    mov  ecx, wdt   // width
    
@@pixel_loop:
    xor  eax,  eax
    mov  ax,   [esi]
    mov  bl,   al

    // bl = __BBBBBb
    shl  bl,   1
    and  bl,   $3F

    // ah = __RRRRRr
    shr  ax,   3
    shl  ah,   1
    
    // al = __GGGGGG
    shr  al,   2

    add  al,   ah
    add  al,   bl
    mov  [edi],  al

    add  esi,  2
    inc  edi

    dec  ecx
    jnz  @@pixel_loop

    add  esi, snl   // source next line
    add  edi, tnl   // target next line
    dec  edx
    jnz  @@line_loop
END;

