{
   muu TrueColor library for TMT Pascal - Format unit
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.       
}

UNIT mFormat;

INTERFACE
{$R-,I+,Q-}

USES Nova,Memory;

TYPE PixelFormat =  Nova_PixelFormat;

    // Format record
TYPE Format =   RECORD
                  Pixelformat:  PixelFormat;
                  Direct:       boolean;
                  Palette:      VPaletteT;
                END;


    // pack 32bpp pixel -> x
FUNCTION Pack16(pix32:dword):word;
FUNCTION Pack15(pix32:dword):word;
FUNCTION Pack8(pix32:dword):byte;
    // unpack x pixel -> 32bpp
FUNCTION Unpack16(pix16:word):dword;
FUNCTION Unpack15(pix15:word):dword;

// --------------------------------------------------------------------------
FUNCTION Format_Pixelformat(m_pixelformat:PixelFormat): Format;
FUNCTION Format_Format(m_format:Format): Format;
FUNCTION Format_String(m_string:string): Format;
FUNCTION Format_Numeric(m_depth:byte): Format;
    OVERLOAD Format = Format_Pixelformat;
    OVERLOAD Format = Format_Format;
    OVERLOAD Format = Format_String;
    OVERLOAD Format = Format_Numeric;

    // Special palette initializers
FUNCTION Format_VPalette(m_vpal:VPaletteT): Format;
    OVERLOAD Format = Format_VPalette;

    // Format information access
FUNCTION Format_Depth(m_format:Format): byte;
FUNCTION Format_Pitch(m_format:Format): byte;
    OVERLOAD Depth = Format_Depth;
    OVERLOAD Pitch = Format_Pitch;

IMPLEMENTATION

    // pixel packers/unpackers
{$I include\pixel.pas}

FUNCTION Format_Pixelformat;
BEGIN
  result.Pixelformat := m_Pixelformat;
  result.Direct := (m_Pixelformat <> VPAL);
  IF (m_Pixelformat = VPAL) THEN
    memset(ptr(ofs(result.Palette)),sizeof(VpaletteT),0);
END;

FUNCTION Format_Format;
BEGIN
  result.Pixelformat := m_format.Pixelformat;
  result.Direct := (m_format.Pixelformat <> VPAL);
  IF (m_format.Pixelformat = VPAL) THEN
    memcpy(ptr(ofs(m_format.Palette)),ptr(ofs(result.Palette)),sizeof(VPaletteT));
END;

FUNCTION Format_String;
  VAR c:byte := 0;
BEGIN
  FOR c := 1 TO length(m_string) DO
    m_string[c] := upcase(m_string[c]);

  IF (m_string = '32') THEN
    result := Format(BGR8888)
  ELSE IF (m_string = '24') THEN
    result := Format(BGR888)
  ELSE IF (m_string = '16') THEN
    result := Format(BGR565)
  ELSE IF (m_string = '15') THEN
    result := Format(BGR555)
  ELSE IF (m_string = '8') THEN
    result := Format(RGB323)
  ELSE
    result := Format(Unknown);
END;

FUNCTION Format_Numeric;
  CASE (m_depth) OF
    8:  result := Format(VPAL)
    15:  result := Format(BGR555)
    16:  result := Format(BGR565)
    24:  result := Format(BGR888)
    32:  result := Format(BGR8888)

    ELSE
      result := Format(Unknown)
  END;


// --------------------------------------------------------------------------
FUNCTION Format_VPalette;
BEGIN
  result.PixelFormat := VPAL;
  result.Direct := false;
  memcpy(ptr(ofs(m_vpal)),ptr(ofs(result.Palette)),sizeof(VPaletteT));
END;

// --------------------------------------------------------------------------
FUNCTION Format_Depth;
  CASE (m_format.pixelformat) OF
    RGB8888:    result := 32;
    RGB888:     result := 24;
    RGB565:     result := 16;
    RGB555:     result := 15;
    BGR8888:    result := 32;
    BGR888:     result := 24;
    BGR565:     result := 16;
    BGR555:     result := 15;
    RGB323:     result := 8;
    GRAY8:      result := 8;
    DITHER8:    result := 8;
    DITHER15:   result := 15;
    DITHER16:   result := 16;
    VPAL:       result := 8;

    ELSE    result := 0;
  END;

FUNCTION Format_Pitch;
  CASE (m_format.pixelformat) OF
    RGB8888:    result := 4;
    RGB888:     result := 3;
    RGB565:     result := 2;
    RGB555:     result := 2;
    BGR8888:    result := 4;
    BGR888:     result := 3;
    BGR565:     result := 2;
    BGR555:     result := 2;
    RGB323:     result := 1;
    GRAY8:      result := 1;
    DITHER8:    result := 1;
    DITHER15:   result := 2;
    DITHER16:   result := 2;
    VPAL:       result := 1;

    ELSE    result := 0;
  END;

end.
