{
   muu TrueColor library for TMT Pascal - Nova pixel converter&clearer
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

UNIT Nova;

INTERFACE
{$R-,I+,Q-}

    // Here we define which SOURCE pixelformats to support, e.g enabling
    // 32bpp support enables Nova to convert from 32bpp to all other formats.
    // The VPal support is a special palette format and Nova is only able to
    // convert _from_ VPal to any other format.

{$DEFINE SUPPORT_32BPP}
{$DEFINE SUPPORT_24BPP}
{$DEFINE SUPPORT_16BPP}
{$DEFINE SUPPORT_15BPP}
//{$DEFINE SUPPORT_8BPP}
{$DEFINE SUPPORT_VPAL}

USES CPUInfo,Memory;

    // Virtual Palette system
TYPE Channels = (VBlue,VGreen,VRed,VAlpha);
TYPE VPaletteT = ARRAY[BYTE,Channels] OF byte;
     VPalette = ^VpaletteT;

    // converter and clearer type definition
TYPE Nova_Converter =   PROCEDURE(src,tgt:pointer;wdt,hgt,snl,tnl:dword);
TYPE Nova_Clearer =     PROCEDURE(tgt:pointer;col,wdt,hgt,nl:dword);

    // color format support
TYPE Nova_PixelFormat = (RGB8888,RGB888,RGB565,RGB555,
                         BGR8888,BGR888,BGR565,BGR555,RGB323,
                         GRAY8,DITHER8,DITHER15,DITHER16,
                         VPAL,
                         Unknown);

TYPE Nova_CPUmode = (Nx86,Nfpu,Nmmx);

    // dithering variables
    {$I include\nova\dtables.pas}

// get converter/clearer
FUNCTION Nova_getConverter(VAR gcpu:cpuinfoT;srcFM,tgtFM:nova_PixelFormat): Nova_converter;
FUNCTION Nova_getClearer(VAR gcpu:cpuinfoT;Size:byte): Nova_clearer;

// Virtual Palette routines
PROCEDURE Nova_setVPal(VAR m_vpal: VPaletteT);


IMPLEMENTATION

TYPE Nova_converterlist =   ARRAY[nova_PixelFormat] OF Nova_converter;
TYPE Nova_clearerlist =     ARRAY[1..4] OF Nova_clearer;

VAR Nova_VPal:  VPalette := NIL;
    Nova_mCPU:  Nova_CPUmode := Nx86;

// Converters and Clearer lists
// --------------------------------------------------------------------------
    // direct converters
    {$I include\nova\x86_dir.pas}

{$IFDEF SUPPORT_32BPP}
    // 32BPP -> X
    {$I include\nova\x86_p32.pas}
    {$I include\nova\x86_d32.pas}
CONST Nova_x86_BGR8888_converters: Nova_converterlist = // x86 converters
     (Nx86_S32_D32_SWAP_RB,
      Nx86_S32_D24_SWAP_RB,
      Nx86_S32_D16_SWAP_RB,
      Nx86_S32_D15_SWAP_RB,
      Nx86_DIRECT4,
      Nx86_S32_D24,
      Nx86_S32_D16,
      Nx86_S32_D15,
      Nx86_S32_D8,
      Nx86_S32_D8_GRAY,
      Dx86_S32_D8,
      Dx86_S32_D15,
      Dx86_S32_D16,
      NIL,
      NIL);

CONST Nova_x86_RGB8888_converters: Nova_converterlist = // x86 converters
     (Nx86_DIRECT4,
      Nx86_S32_D24,
      Nx86_S32_D16,
      Nx86_S32_D15,
      Nx86_S32_D32_SWAP_RB,
      Nx86_S32_D24_SWAP_RB,
      Nx86_S32_D16_SWAP_RB,
      Nx86_S32_D15_SWAP_RB,
      Nx86_S32_D8_SWAP_RB,
      Nx86_S32_D8_GRAY,
      Dx86_S32_D8_SWAP_RB,
      Dx86_S32_D15_SWAP_RB,
      Dx86_S32_D16_SWAP_RB,
      NIL,
      NIL);
{$ENDIF}

{$IFDEF SUPPORT_24BPP}
    // 24BPP -> X
    {$I include\nova\x86_p24.pas}
    {$I include\nova\x86_d24.pas}
CONST Nova_x86_BGR888_converters: Nova_converterlist = // x86 converters
     (Nx86_S24_D32_SWAP_RB,
      Nx86_S24_D24_SWAP_RB,
      Nx86_S24_D16_SWAP_RB,
      Nx86_S24_D15_SWAP_RB,
      Nx86_S24_D32,
      Nx86_DIRECT3,
      Nx86_S24_D16,
      Nx86_S24_D15,
      Nx86_S24_D8,
      Nx86_S24_D8_GRAY,
      Dx86_S24_D8,
      Dx86_S24_D15,
      Dx86_S24_D16,
      NIL,
      NIL);

CONST Nova_x86_RGB888_converters: Nova_converterlist = // x86 converters
     (Nx86_S24_D32,
      Nx86_DIRECT3,
      Nx86_S24_D16,
      Nx86_S24_D15,
      Nx86_S24_D32_SWAP_RB,
      Nx86_S24_D24_SWAP_RB,
      Nx86_S24_D16_SWAP_RB,
      Nx86_S24_D15_SWAP_RB,
      Nx86_S24_D8_SWAP_RB,
      Nx86_S24_D8_GRAY,
      Dx86_S24_D8_SWAP_RB,
      Dx86_S24_D15_SWAP_RB,
      Dx86_S24_D16_SWAP_RB,
      NIL,
      NIL);
{$ENDIF}

{$IFDEF SUPPORT_16BPP}
    // 16BPP -> X
    {$I include\nova\x86_p16.pas}
CONST Nova_x86_BGR565_converters: Nova_converterlist = // x86 converters
     (Nx86_S16_D32_SWAP_RB,
      Nx86_S16_D24_SWAP_RB,
      Nx86_S16_D16_SWAP_RB,
      Nx86_S16_D15_SWAP_RB,
      Nx86_S16_D32,
      Nx86_S16_D24,
      Nx86_DIRECT2,
      Nx86_S16_D15,
      Nx86_S16_D8,
      Nx86_S16_D8_GRAY,
      Nx86_S16_D8,
      Nx86_S16_D15,
      Nx86_DIRECT2,
      NIL,
      NIL);

CONST Nova_x86_RGB565_converters: Nova_converterlist = // x86 converters
     (Nx86_S16_D32,
      Nx86_S16_D24,
      Nx86_DIRECT2,
      Nx86_S16_D15,
      Nx86_S16_D32_SWAP_RB,
      Nx86_S16_D24_SWAP_RB,
      Nx86_S16_D16_SWAP_RB,
      Nx86_S16_D15_SWAP_RB,
      Nx86_S16_D8_SWAP_RB,
      Nx86_S16_D8_GRAY,
      Nx86_S16_D8_SWAP_RB,
      Nx86_S16_D15_SWAP_RB,
      Nx86_S16_D16_SWAP_RB,
      NIL,
      NIL);
{$ENDIF}

{$IFDEF SUPPORT_15BPP}
    // 16BPP -> X
    {$I include\nova\x86_p15.pas}
CONST Nova_x86_BGR555_converters: Nova_converterlist = // x86 converters
     (Nx86_S15_D32_SWAP_RB,
      Nx86_S15_D24_SWAP_RB,
      Nx86_S15_D16_SWAP_RB,
      Nx86_S15_D15_SWAP_RB,
      Nx86_S15_D32,
      Nx86_S15_D24,
      Nx86_S15_D16,
      Nx86_DIRECT2,
      Nx86_S15_D8,
      Nx86_S15_D8_GRAY,
      Nx86_S15_D8,
      Nx86_DIRECT2,
      Nx86_S15_D16,
      NIL,
      NIL);

CONST Nova_x86_RGB555_converters: Nova_converterlist = // x86 converters
     (Nx86_S15_D32,
      Nx86_S15_D24,
      Nx86_S15_D16,
      Nx86_DIRECT2,
      Nx86_S15_D32_SWAP_RB,
      Nx86_S15_D24_SWAP_RB,
      Nx86_S15_D16_SWAP_RB,
      Nx86_S15_D15_SWAP_RB,
      Nx86_S15_D8_SWAP_RB,
      Nx86_S15_D8_GRAY,
      Nx86_S15_D8_SWAP_RB,
      Nx86_S15_D15_SWAP_RB,
      Nx86_S15_D16_SWAP_RB,
      NIL,
      NIL);
{$ENDIF}

{$IFDEF SUPPORT_VPAL}
    // VPALETTE -> X
    {$I include\nova\x86_vpal.pas}
CONST Nova_x86_VPAL_converters: Nova_converterlist = // x86 converters
     (Nx86_VPAL_D32_SWAP_RB,
      Nx86_VPAL_D24_SWAP_RB,
      Nx86_VPAL_D16_SWAP_RB,
      Nx86_VPAL_D15_SWAP_RB,
      Nx86_VPAL_D32,
      Nx86_VPAL_D24,
      Nx86_VPAL_D16,
      Nx86_VPAL_D15,
      Nx86_VPAL_D8,
      Nx86_VPAL_D8_GRAY,
      Nx86_VPAL_D8,
      Nx86_VPAL_D15,
      Nx86_VPAL_D16,
      Nx86_DIRECT1, // no palette conversion
      NIL);
{$ENDIF}

    // clearer routines
    {$I include\nova\x86_clr.pas}

CONST Nova_x86_clearers: Nova_clearerlist =    // x86 clearers
    (Nx86_CLEAR_1BYTE,
     Nx86_CLEAR_2BYTE,
     Nx86_CLEAR_3BYTE,
     Nx86_CLEAR_4BYTE);

    {$I include\nova\mmx_clr.pas}
CONST Nova_mmx_clearers: Nova_clearerlist =    // MMX clearers
    (Nx86_CLEAR_1BYTE,
     Nx86_CLEAR_2BYTE,
     Nx86_CLEAR_3BYTE,
     Nmmx_CLEAR_4BYTE);

// User interface routines
// --------------------------------------------------------------------------
FUNCTION Nova_getConverter;
  CASE (srcFM) OF
{$IFDEF SUPPORT_32BPP}
    BGR8888:    result := nova_x86_BGR8888_converters[tgtFM]
    RGB8888:    result := nova_x86_RGB8888_converters[tgtFM]
{$ENDIF}

{$IFDEF SUPPORT_24BPP}
    BGR888:     result := nova_x86_BGR888_converters[tgtFM]
    RGB888:     result := nova_x86_RGB888_converters[tgtFM]
{$ENDIF}

{$IFDEF SUPPORT_16BPP}
    BGR565:     result := nova_x86_BGR565_converters[tgtFM]
    RGB565:     result := nova_x86_RGB565_converters[tgtFM]
{$ENDIF}

{$IFDEF SUPPORT_15BPP}
    BGR555:     result := nova_x86_BGR555_converters[tgtFM]
    RGB555:     result := nova_x86_RGB555_converters[tgtFM]
{$ENDIF}

{$IFDEF SUPPORT_VPAL}
    VPAL:       result := nova_x86_VPAL_converters[tgtFM]
{$ENDIF}
    
    Unknown:    result := NIL;
   ELSE
    result := NIL;
  END;
                                   
FUNCTION Nova_getClearer;
  CASE (Nova_mCPU) OF
    Nx86:   result := nova_x86_clearers[Size];
    Nfpu:   result := nova_x86_clearers[Size];
    Nmmx:   result := nova_mmx_clearers[Size];
  END;

// Virtual palette routines
// --------------------------------------------------------------------------
PROCEDURE Nova_setVPal;
  memcpy(ptr(ofs(m_vpal)),Nova_VPal,sizeof(VPaletteT));


// Initialization routines
// --------------------------------------------------------------------------
    {$I include\nova\dmatrix.pas}

begin
    // create dithering matrices
  Dither_SetupMatrices;
    
    // initialize and clear virtual palette
  new(Nova_VPal);
  memset(Nova_VPal,sizeof(VPaletteT),0);

    // set CPU mode
  IF (cpu_info.mmx) THEN
    Nova_mCPU := Nmmx
  ELSE IF (cpu_info.fpu) AND
          (cpu_info.family > 4) THEN
    Nova_mCPU := Nfpu
  ELSE
    Nova_mCPU := Nx86;
end.
