{
   muu Library for TMT Pascal - Various detection routines
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.       
}

UNIT Detect;

INTERFACE
{$R-,I+,Q-}

    // get EMS version [4bit_minor:4bit_major] (returns -1 if not installed)
FUNCTION EMSVersion: shortint;
    // get Windows version [8bit_minor:8bit_major] (returns -1 if not present)
FUNCTION WINVersion: integer;

IMPLEMENTATION

FUNCTION EMSVersion; assembler;
  VAR handle:   word := 0;
ASM
    // open file/device "EMMXXXX0"
  mov  ah,  $3D
  mov  al,  $00 // read only mode
  mov  edx, OFFSET @@emm_file_name
  int  $21
  jnc  @@checkdevice

    // unable to open device/file; no EMM present
  jmp  @@fail

@@checkdevice:
  mov  [handle],  ax

    // determine if "EMMXXXX0" is a device
  mov  ah,  $44
  mov  al,  $00 
  mov  bx,  [handle]
  int  $21  // IOCTL function 0
  jc   @@close_and_fail
    // check for bit 7 in status, should be on for char.device
  test dx,  $80
  jc   @@close_and_fail

    // get device status
  mov  ah,  $44
  mov  al,  $07
  mov  bx,  [handle]
  int  $21  // IOCTL function 7
  jc   @@close_and_fail
    // al should be $FF for a char.device
  cmp  al,  $FF
  jne  @@close_and_fail

    // close handle
  mov  ah,  $3E
  mov  bx,  [handle]
  int  $21
    // get EMM version
  mov  ah,  $46
  int  $67
  jmp  @@done

@@emm_file_name:    // ASCIIZ (zero terminated) filename
  db 'EMMXXXX0',0

    // close handle and fail
@@close_and_fail:
  mov  ah,  $3E
  mov  bx,  [handle]
  int  $21

@@fail:
  mov  al,  -1

@@done:
END;


FUNCTION WINVersion; assembler;
ASM
    // check for windows real mode
  mov  ax,  $4680
  int  $2F
  cmp  ax,  0
  je   @@detected

    // check for windows enhanced mode
  mov  ax,  $1600
  int  $2F
  cmp  al,  $00
  je   @@notpresent
  cmp  al,  $01
  je   @@notpresent
  cmp  al,  $80
  je   @@notpresent
  cmp  al,  $FF
  je   @@notpresent

    // get windows version and exit
@@detected:
  mov  ax,  $1600
  int  $2F
  jmp  @@done

@@notpresent:
  mov  ax,  -1

@@done:
END;



end.
