{
   muu Library for TMT Pascal - High resolution timer v2.00
   Copyright (C) 2000 TurboPower Software (from the SWAG package)
   Ported to TMT Pascal & OOPified by Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

UNIT HTimer;

INTERFACE
{$R-,I+,Q-}

USES Error;

CONST TimerResolution = 1193181.667;

TYPE Timer = ^TimerT;
     TimerT =   OBJECT

                 public

                  CONSTRUCTOR Create;
                  DESTRUCTOR Destroy;

                  PROCEDURE Start;
                  PROCEDURE Stop;
                  PROCEDURE Reset;

                  FUNCTION Elapsed: real;
                  PROCEDURE Wait(microsecs:longint);

                 private

                  FUNCTION Cardinal(l:longint): real;
                  FUNCTION Time: longint;

                  t_active: boolean;
                  t_start:  longint;
                  t_stop:   longint;
                END;

  {==========================================================================}

IMPLEMENTATION

VAR delta:  longint := 0;

CONSTRUCTOR TimerT.Create;
BEGIN
  t_start := 0;
  t_stop := 0;
  t_active := false;
END;

DESTRUCTOR TimerT.Destroy;
BEGIN
  t_start := 0;
  t_stop := 0;
  t_active := false;
END;

PROCEDURE TimerT.Start;
BEGIN
  IF (t_active) THEN
    fail('HTimer:  Timer already started');

  t_start := Time;
  t_stop := 0;
  t_active := true;
END;

PROCEDURE TimerT.Stop;
BEGIN
  IF (NOT t_active) THEN
    fail('HTimer:  Timer not started');

  t_stop := Time;
  t_active := false;
END;

PROCEDURE TimerT.Reset;
BEGIN
  t_start := Time;
  t_stop := 0;
  t_active := true;
END;

FUNCTION TimerT.Elapsed;
  IF (t_active) THEN
    result := 1000.0*Cardinal(Time-(t_start+delta))/TimerResolution
  ELSE
    result := 1000.0*Cardinal(t_stop-(t_start+delta))/TimerResolution;

PROCEDURE TimerT.Wait;
 VAR w_start: longint := Time;
  WHILE ( (w_start+microsecs) > Time) DO;

FUNCTION TimerT.Cardinal;
 IF (l < 0) THEN
   result := 4294967296.0+l
  ELSE
   result := l;

FUNCTION TimerT.Time: longint;
  VAR t:    longint := 0;
BEGIN
  ASM
    cli
    mov  dx,    $20
    mov  al,    $0A
    out  dx,    al
    mov  al,    $00
    out  $43,   al
    in   al,    dx
    mov  di,    ax
    in   al,    $40
    mov  bl,    al
    in   al,    $40
    mov  bh,    al
    not  bx
    in   al,    $21
    mov  si,    ax
    mov  al,    $0FF
    out  $21,   al
    mov  eax,   $400
    mov  esi,   eax
    mov  dx,    [esi+$6C]
    mov  ax,    si
    out  $21,   al
    sti
    mov  ax,    di
    test al,    $01
    jz   @@done
    cmp  bx,    $FF
    ja   @@done
    inc  dx

  @@done:
    shl  edx,   16
    mov  dx,    bx
    mov  t,     edx
  END;

  result := t;
END;


    // -----------------------------------------
    // common timer resolution setup
    // -------------------------------------

    // Calibrate timer routines to computer speed
PROCEDURE Calibrate;
VAR i: dword := 0;
    L1,L2,diff: longint := 0;
    t:  Timer;
BEGIN
  new(t,Create);

  delta := maxint;
  FOR i := 0 TO 1999 DO
  BEGIN
    L1 := t^.Time;
    L2 := t^.Time;

        // wrapped
    IF (L1>L2) THEN
      Continue;

        // use minimum difference
    diff := L2-L1;
   
    IF (diff < delta) THEN
      delta := diff;
  END;

  dispose(t,Destroy);
END;

    // Reprogram the timer chip to allow 1 microsecond resolution
PROCEDURE initializetimer;
BEGIN
    // select timer mode 2, read/write channel 0
  port[$43] := $34; // 00110100b
  inline($EB/$00);  // jmp short $+2, delay
  port[$40] := $00; // LSB = 0
  inline($EB/$00);  // jmp short $+2, delay
  port[$40] := $00; // MSB = 0
END;

    // Restore the timer chip to its normal state
PROCEDURE restoretimer;
BEGIN
    // select timer mode 3, read/write channel 0
  port[$43] := $36; // 00110110b
  inline($EB/$00);  // jmp short $+2, delay
  port[$40] := $00; // LSB = 0
  inline($EB/$00);  // jmp short $+2, delay
  port[$40] := $00; // MSB = 0
END;

VAR oldexitproc:    pointer := NIL;

  {$F+}
  PROCEDURE tptimer_exitproc;
  BEGIN
    exitproc := oldexitproc;
    restoretimer;
  END;
  {$F-}

BEGIN
    // set up timer exit handler
  oldexitproc := exitproc;
  exitproc := @tptimer_exitproc;

    // reprogram the timer chip
  initializetimer;
  Calibrate;
END.
