{
   muu TrueColor library for TMT Pascal - Advanced memory management unit
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.
}

UNIT Memory;

INTERFACE
{$R-,I+,Q-}

USES CPUInfo,Error;

    // Experimental features
    //
    // WARNING: These features is known to create strange effects which can 
    //          cause your computer to reboot. Therefore they should always
    //          be turned off when not debugging the actual features.

//{$DEFINE NEW_MEMORY_ALLOCATION}   // <default off>
//{$DEFINE ALIGN_MEMORY}            // <default off>

// --------------------------------------------------------------------------
CONST _nearest_alignment =  32; // bytes to align memory to

TYPE _alloclistP =  ^_alloclistT;
     _alloclistT =  RECORD
                      memory_allocated: pointer;    // non-aligned memory
                      memory_realsize:  dword;      // allocated size
                      memblock_handle:  dword;      // memory handle
                      memory_aligned:   pointer;    // aligned memory
                      next,prev:    _alloclistP;    // next+previous item
                    END;

{$IFDEF NEW_MEMORY_ALLOCATION}
PROCEDURE getmem_aligned(VAR data:pointer;size:dword);  // allocate memory
PROCEDURE freemem_aligned(VAR data:pointer;size:dword); // free memory
  OVERLOAD getmem = getmem_aligned;
  OVERLOAD freemem = freemem_aligned;
{$ENDIF}


// --------------------------------------------------------------------------

PROCEDURE memcpy_dword(source,target:pointer;size:dword);
  OVERLOAD memcpy = memcpy_dword;

    // overloads which should make it possible to pass a single byte
PROCEDURE memset_dword(target:pointer;size:dword;fill:dword);
PROCEDURE memset_word(target:pointer;size:dword;fill:word);
PROCEDURE memset_byte(target:pointer;size:dword;fill:byte);
  OVERLOAD memset = memset_byte;
  OVERLOAD memset = memset_word;
  OVERLOAD memset = memset_dword;

// --------------------------------------------------------------------------
IMPLEMENTATION

TYPE _memcpy_procedure = PROCEDURE(source,target:pointer;size:dword);
     _memset_procedure = PROCEDURE(target:pointer;size,fill:dword);

VAR memcpy_opt: _memcpy_procedure := NIL;   // copy memory
    memset_opt: _memset_procedure := NIL;   // fill memory

PROCEDURE memcpy_dword(source,target:pointer;size:dword);
  memcpy_opt(source,target,size);

PROCEDURE memset_dword(target:pointer;size:dword;fill:dword);
  memset_opt(target,size,fill);
PROCEDURE memset_word(target:pointer;size:dword;fill:word);
  memset_opt(target,size,fill+(fill shl 16));
PROCEDURE memset_byte(target:pointer;size:dword;fill:byte);
  memset_opt(target,size,fill+(fill shl 8));

// --------------------------------------------------------------------------
VAR malloc_first,malloc_last:   _alloclistP := NIL; // internal memory lists

PROCEDURE getmem_aligned(VAR data:pointer;size:dword);
  VAR gdata:    pointer := NIL;
      gsize:    dword := 0;
      ghandle:  dword := 0;
BEGIN
{$IFDEF ALIGN_MEMORY}
  gsize := size+(_nearest_alignment shl 3);
{$ELSE}
  gsize := size;
{$ENDIF}

  ASM
    pushad

    mov ax,  $501
    mov bx,  word [gsize+2]
    mov cx,  word [gsize]
    int $31
    jc  @@nomem

    // get memory address
    mov word [gdata],  cx
    mov word [gdata+2],  bx
    // get memory block handle
    mov dx,  di
    mov word [ghandle],  dx
    mov dx,  si
    mov word [ghandle+2],  dx

  @@nomem:
    popad
  END;

    // check if anything was allocated - if not, fail.
  IF (gdata = NIL) THEN
    fail('muuLib:   Memory allocation failed (not enough memory)');

{$IFDEF ALIGN_MEMORY}
  data := ptr((ofs(gdata^)+(_nearest_alignment shl 3)) and $FFFFFF00);
{$ELSE}
  data := gdata;
{$ENDIF}

    // save memory pointer into the linked list for deallocation
  IF (malloc_first = NIL) THEN
  BEGIN
    new(malloc_first);
    malloc_first^.memory_aligned := data;
    malloc_first^.memory_allocated := gdata;
    malloc_first^.memory_realsize := gsize;
    malloc_first^.memblock_handle := ghandle;
    malloc_first^.next := NIL;
    malloc_first^.prev := NIL;
    malloc_last := malloc_first;
  END ELSE
  BEGIN
    new(malloc_last^.next);
    malloc_last^.next^.memory_aligned := data;
    malloc_last^.next^.memory_allocated := gdata;
    malloc_last^.next^.memory_realsize := gsize;
    malloc_last^.next^.memblock_handle := ghandle;
    malloc_last^.next^.prev := malloc_last;
    malloc_last := malloc_last^.next;
    malloc_last^.next := NIL;
  END;
END;

PROCEDURE freemem_aligned(VAR data:pointer;size:dword);
  VAR gdata:    pointer := NIL;
      gsize:    dword := 0;
      ghandle:  dword := 0;
BEGIN
  IF (malloc_first = NIL) THEN  // check if anything is allocated, else exit
    exit;

    // locate data in data list and deallocate memory
  DECLARE VAR mlp:  _alloclistP := malloc_first;
  WHILE (mlp <> malloc_last^.next) DO
  BEGIN
    IF (mlp^.memory_aligned = data) THEN
    BEGIN
      ghandle := mlp^.memblock_handle;
      ASM
        pushad
        mov ax,  $502
        mov dx,  word [ghandle]
        mov di,  dx
        mov dx,  word [ghandle+2]
        mov si,  dx
        int $31
        popad
      END;
          
      IF (mlp^.prev <> NIL) THEN
        mlp^.prev^.next := mlp^.next;

      IF (mlp^.next <> NIL) THEN
        mlp^.next^.prev := mlp^.prev;

      DECLARE VAR tlp:  _alloclistP;
      BEGIN
        mlp^.memory_aligned := NIL;
        mlp^.memory_allocated := NIL;
        mlp^.memory_realsize := 0;
        mlp^.memblock_handle := 0;
        dispose(mlp);
      END;
      
      data := NIL;
      exit;
    END;

    // get next item in list
    mlp := mlp^.next;
  END;
END;


// 386 optimized memory management code
// --------------------------------------------------------------------------
PROCEDURE memcpy_386(source,target:pointer;size:dword); assembler;
ASM
    mov  esi,  source
    mov  edi,  target
    mov  ecx,  size
    mov  edx,  size
    shr  ecx,  2
    jz   @@no_opt

    align 4
    align 4
@@move_opt:
    rep  movsd

@@no_opt:
    and  edx,  3
    jz   @@no_comp

@@move_comp:    // moves remaining bytes which doesn't match a dword
    mov  ecx,  edx
    rep  movsb

@@no_comp:
END;

PROCEDURE memset_386(target:pointer;size,fill:dword); assembler;
ASM
    mov  edi,  target
    mov  eax,  fill
    mov  ecx,  size
    mov  edx,  size
    shr  ecx,  2
    jz   @@no_opt

    align 4
    align 4
@@fill_opt:
    rep  stosd

@@no_opt:
    and  edx,  3
    jz   @@no_comp

@@fill_comp:
    mov  [edi],  al
    ror  eax,  8
    inc  edi
    dec  edx
    jnz  @@fill_comp

@@no_comp:
END;

// Pentium optimized memory management code (FPU)
// --------------------------------------------------------------------------
PROCEDURE memcpy_FPU(source,target:pointer;size:dword); assembler;
ASM
    mov  esi,  source
    mov  edi,  target
    mov  ecx,  size
    mov  edx,  size
    shr  ecx,  4
    jz   @@no_opt

        // initialize FPU
    fninit

    align 4
    align 4
@@move_opt:
    fild QWORD [esi]
    fild QWORD [esi+8]
    fxch
    fistp QWORD [edi]
    fistp QWORD [edi+8]
    
    add  esi,  16
    add  edi,  16
    dec  ecx
    jnz  @@move_opt

        // wait for unfinished FPU instructions
    fwait

@@no_opt:
    and  edx,  15
    jz   @@no_comp

@@move_comp:    // moves remaining bytes which doesn't match 2xqword
    mov  ecx,  edx
    rep  movsb

@@no_comp:
END;

PROCEDURE memset_FPU(target:pointer;size,fill:dword); assembler;
ASM
    mov  eax,  fill
    mov  esi,  @@filler
    mov  [esi],  eax
    mov  [esi+4],  eax

    mov  edi,  target
    mov  ecx,  size
    mov  edx,  size
    shr  ecx,  4
    jz   @@no_opt

        // initialize FPU
    fninit

    align 4
    align 4
@@move_opt:
    fild QWORD [esi]
    fild QWORD [esi]

    fistp QWORD [edi]
    fistp QWORD [edi+8]
    
    add  edi,  16
    dec  ecx
    jnz  @@move_opt

        // wait for unfinished FPU instructions
    fwait

@@no_opt:
    and  edx,  15
    jz   @@no_comp

@@fill_comp:
    mov  [edi],  al
    ror  eax,  8
    inc  edi
    dec  edx
    jnz  @@fill_comp

@@no_comp:
    jmp  @@m_end

@@filler:
    dd  $00000000
    dd  $00000000
    dd  $00000000
    dd  $00000000

@@m_end:

END;


// MMX/PII optimized memory management code
// --------------------------------------------------------------------------
PROCEDURE memcpy_MMX(source,target:pointer;size:dword); assembler;
ASM
    mov  esi,  source
    mov  edi,  target
    mov  ecx,  size
    mov  edx,  size
    shr  ecx,  4
    jz   @@no_opt

        // make sure FPU isn't running
    fwait

    align 4
    align 4
@@move_opt:
    movq mm0,  [esi]
    movq mm1,  [esi+8]

    movq [edi],  mm0
    movq [edi+8],  mm1
    
    add  esi,  16
    add  edi,  16
    dec  ecx
    jnz  @@move_opt

        // shut down MMX
    emms

@@no_opt:
    and  edx,  15
    jz   @@no_comp

@@move_comp:    // moves remaining bytes which doesn't match 2xqword
    mov  ecx,  edx
    rep  movsb

@@no_comp:
END;

PROCEDURE memset_MMX(target:pointer;size,fill:dword); assembler;
ASM
    mov  eax,  fill
    mov  esi,  @@filler
    mov  [esi],  eax
    mov  [esi+4],  eax

    mov  edi,  target
    mov  ecx,  size
    mov  edx,  size
    shr  ecx,  4
    jz   @@no_opt

        // make sure FPU isn't running
    fwait

    movq mm0,  [esi]

    align 4
    align 4
@@move_opt:
    movq [edi],  mm0
    movq [edi+8],  mm0
    
    add  edi,  16
    dec  ecx
    jnz  @@move_opt

        // shut down MMX
    emms

@@no_opt:
    and  edx,  15
    jz   @@no_comp

@@fill_comp:
    mov  [edi],  al
    ror  eax,  8
    inc  edi
    dec  edx
    jnz  @@fill_comp

@@no_comp:
    jmp  @@m_end


@@filler:
    dd  $00000000
    dd  $00000000
    dd  $00000000
    dd  $00000000

@@m_end:

END;


//---------------------------------------------------------------------------
PROCEDURE memory_initprocedures;
BEGIN
  IF (cpu_info.mmx) THEN
  BEGIN
    memcpy_opt := memcpy_MMX;
    memset_opt := memset_MMX;
  END
  ELSE IF (cpu_info.fpu) AND
          (cpu_info.family > 4) THEN
  BEGIN
    memcpy_opt := memcpy_FPU;
    memset_opt := memset_FPU;
  END
  ELSE BEGIN
    memcpy_opt := memcpy_386;
    memset_opt := memset_386;
  END;
END;


// --------------------------------------------------------------------------
begin
  memory_initprocedures;    // initialize memory functions at startup
end.
