{
   muu TrueColor library for TMT Pascal - VESA VBE interface object
   Copyright (C) 1999 Tor Arne Gustad (anden@online.no)

   This source code is licensed under the GNU Public License.
   See the GNU General Public License for more details.       
}

UNIT VBE;

INTERFACE
{$R-,I+,Q-}

USES VBECore,VBETypes,DPMI,Memory;

TYPE VESAinterface =    ^VESAinterfaceT;
     VESAinterfaceT =   OBJECT
                         public
                          CONSTRUCTOR open;
                          DESTRUCTOR close;
                          FUNCTION error:   boolean;
                          PROCEDURE seterror(state:boolean);

                          PROCEDURE getinfo;
                          PROCEDURE getmodeinfo(mode:word);
                          PROCEDURE setmode(mode:word);
                          FUNCTION getmode: word;

                          PROCEDURE savestate(what:word);
                          PROCEDURE restorestate;

                          PROCEDURE setwindow(wn_N:VBEwindowT;wndw:word);
                          FUNCTION getwindow(wn_N:VBEwindowT):  word;

                          PROCEDURE setdisplaystart(xpan,ypan:integer);
                          PROCEDURE getdisplaystart(VAR xpan,ypan:integer);

                          PROCEDURE setDACwidth(width:byte);
                          FUNCTION getDACwidth: byte;

                          PROCEDURE setpalette(VAR pal:VBE_VESApalette;first,count:word);
                          PROCEDURE getpalette(VAR pal:VBE_VESApalette;first,count:word);

                         public
                          vesainfo: VBE_VESAinformation;
                          modeinfo: VBE_modeinformation;

                         private
                          v_state:  pointer;    // saved VESA state
                          a_state:  word;       // state attributes
                          v_mode:   word;       // current mode set
                          _state:   shortint;   // interface state

                          seg_16:   word;
                          ofs_32:   dword;
                        END;


// --------------------------------------------------------------------------
IMPLEMENTATION

CONSTRUCTOR VESAinterfaceT.open;
BEGIN
  seg_16 := DOSMemoryAlloc(4096);
  ofs_32 := seg_16*$10;

  new(vesainfo);
  memset(vesainfo,sizeof(VBE_VESAinformationT),0);
  new(modeinfo);
  memset(modeinfo,sizeof(VBE_modeinformationT),0);

  v_state := NIL;
  a_state := 0;
  v_mode := 0;
  _state := 0;
END;

DESTRUCTOR VESAinterfaceT.close;
BEGIN
  ofs_32 := 0;
  DOSMemoryFree(seg_16);
  seg_16 := 0;

  dispose(vesainfo);
  dispose(modeinfo);
END;

FUNCTION VESAinterfaceT.error;
  result := boolean(_state <> 0);   // get state from last function

PROCEDURE VESAinterfaceT.seterror(state:boolean);
  _state := byte(state);

PROCEDURE VESAinterfaceT.getinfo;
BEGIN
  memset(ptr(ofs_32),1024,0);
  memset(ptr(ofs_32),4,$32454256);  // try to enable VESA2.0 (or above)
    
  _state := VBE_getinfo(seg_16);
  IF (_state = 0) THEN
     memcpy(ptr(ofs_32),vesainfo,sizeof(VBE_VESAinformationT))
  ELSE BEGIN    // if VESA2.0 failed, try plain VESA1.x instead (clear lists)
    memset(ptr(ofs_32),1024,0);
    _state := VBE_getinfo(seg_16);
    memcpy(ptr(ofs_32),vesainfo,sizeof(VBE_VESAinformationT));
  END;
END;

PROCEDURE VESAinterfaceT.getmodeinfo(mode:word);
BEGIN
  memset(ptr(ofs_32),1024,0);   // clear buffer or else some VESA controllers
                                // will refuse to write anything to it

  _state := VBE_getmodeinfo(seg_16,mode);   // get mode information

  IF (_state = 0) THEN
    memcpy(ptr(ofs_32),modeinfo,sizeof(VBE_modeinformationT));  // return

    // retrieve VESA information again (for some reason needed..)
  self.getinfo;
END;

PROCEDURE VESAinterfaceT.setmode;
BEGIN
  IF (mode <> v_mode) THEN  // check if information for this mode is loaded
    getmodeinfo(mode);  // if not, get mode information

  IF (_state = 0) THEN
    _state := VBE_setmode(mode);    // try to set VESA mode

  IF (_state = 0) THEN
    v_mode := mode;
END;

FUNCTION VESAinterfaceT.getmode: word;
  _state := VBE_getmode(result);

PROCEDURE VESAinterfaceT.savestate;
  VAR stsz: word := 0;
BEGIN
  _state := VBE_getstatebuffersize(stsz);   // get size of buffer
  IF (_state <> 0) THEN
    exit;
    
  _state := VBE_getstate(seg_16,what);
  IF (_state <> 0) THEN
    exit;

  getmem(v_state,stsz*64);  // allocate memory for state buffer
  memcpy(ptr(ofs_32),v_state,stsz*64);

  a_state := what;
END;

PROCEDURE VESAinterfaceT.restorestate;
  VAR stsz: word := 0;
BEGIN
  IF (a_state = 0) THEN
    exit;

  _state := VBE_getstatebuffersize(stsz);   // get size of buffer
  IF (_state <> 0) THEN
    exit;
    
  memcpy(v_state,ptr(ofs_32),stsz*64);

  _state := VBE_setstate(seg_16,a_state);
  IF (_state <> 0) THEN
    exit;

  freemem(v_state,stsz*64);
  a_state := 0;
END;

PROCEDURE VESAinterfaceT.setwindow;
  _state := VBE_setwindow(wn_N,wndw);

FUNCTION VESAinterfaceT.getwindow(wn_N:VBEwindowT): word;
  _state := VBE_getwindow(wn_N,result);

PROCEDURE VESAinterfaceT.setdisplaystart(xpan,ypan:integer);
  _state := VBE_setdisplaystart(xpan,ypan);

PROCEDURE VESAinterfaceT.getdisplaystart(VAR xpan,ypan:integer);
  _state := VBE_getdisplaystart(xpan,ypan);

PROCEDURE VESAinterfaceT.setDACwidth;
  _state := VBE_setDACwidth(width);

FUNCTION VESAinterfaceT.getDACwidth: byte;
  _state := VBE_getDACwidth(result);

PROCEDURE VESAinterfaceT.setpalette(VAR pal:VBE_VESApalette;first,count:word);
BEGIN
  memcpy(pal,ptr(ofs_32),1024); // move palette data to buffer
  _state := VBE_setpalette(seg_16,first,count);
END;

PROCEDURE VESAinterfaceT.getpalette(VAR pal:VBE_VESApalette;first,count:word);
BEGIN
  _state := VBE_getpalette(seg_16,first,count);
  IF (_state = 0) THEN
    memcpy(ptr(ofs_32),pal,1024);   // get palette data from buffer
END;


// --------------------------------------------------------------------------
end.
